<?php
namespace Drupal\couchbasedrupal;

/**
 * This class alters the log that it's passed to the default drupal logger.
 * @package Drupal\couchbasedrupal
 */
class CouchbaseWatchdog {

  public static $exceptions = [];

  /**
   * Implements hook_watchdog().
   */
  public static function watchdog(array $log_entry) {
    if (!isset($log_entry['variables']['exception']) || !$log_entry['variables']['exception'] instanceof \CouchbaseException) {
      return;
    }

    $exception = &$log_entry['variables']['exception'];
    $message = &$log_entry['message'];

    if (isset($exception->query_string->options['statement'])) {
      try {
        $query_string = $exception->query_string->options['statement'];
        $message .= "Query: $query_string<br>";
      } catch (\Exception $e) {
      }
    }
    // Pass it to the appropiate watchdog handler.
    $modules = module_implements('watchdog');

    if (($key = array_search('couchbasedrupal', $modules)) !== FALSE) {
      unset($modules[$key]);
    }

    unset($log_entry['variables']['exception']);

    foreach ($modules as $module) {
      module_invoke($module, 'watchdog', $log_entry);
    }
  }

  /**
   * Creates unique identifier for exception.
   *
   * @param \CouchbaseException $exception
   *   Exception to process.
   *
   * @return string
   *   Identifier.
   */
  protected static function createIdentifierFromException(\CouchbaseException $exception) {
    $hash = spl_object_hash($exception);
    $identifier = "%uri_%time_%file_%line_%message_%unique";
    $identifier = strtr($identifier, array(
      '%uri' => request_uri(),
      '%time' => REQUEST_TIME,
      '%line' => $exception->getFile(),
      '%file' => $exception->getFile(),
      '%message' => $exception->getMessage(),
    ));

    return md5($identifier) . '_' . $hash;
  }

  /**
   * Identifies exception.
   *
   * This method identifies each exception for validation purposes. A custom
   * watchog handler can recuperate the original exception from the id on
   * the message if the orignal exception is not carried with the
   * log entry.
   *
   * @param \CouchbaseException $exception
   *   Exception to identify.
   *
   * @return string
   *   Identifier.
   */
  public static function identifyException(\CouchbaseException $exception) {
    $exception_identifier = static::createIdentifierFromException($exception);
    static::$exceptions[$exception_identifier] = $exception;
    return $exception_identifier;
  }
}